from __future__ import division

"""Demonstrate GTP commands implemented in Python.

You can run this from the console or under gogui.

"""

import pyego
from pyego import BOARD_SIZE, BLACK, WHITE, Vertex

class DemoCommands(pyego.GtpPy):

    def __init__(self, stack_board):
        self.stack = stack_board

    def get_command_names(self):
        return ["errors", "altitude"]

    def exec_command(self, cmd, params):
        return getattr(self, "do_"+cmd)(params)

    def do_errors(self, params):
        """Demonstrate the error-handling alternatives"""
        if len(params) != 1:
           raise pyego.PyGtpSyntaxError
        param = params[0]
        if param == "msg":
            raise pyego.PyGtpError("failure message")
        if param == "quiet":
            raise pyego.PyGtpError
        if param == "num":
            raise pyego.PyGtpError(3)
        if param == "internal":
            3 / 0
        if param == "sleep":
            # Try Ctrl-C here
            import time
            time.sleep(5)
            return "slept for 5 seconds"
        if param == "newlines":
            return "xx\n\nyy"
        return "errors called with parameter %s" % param

    def do_altitude(self, params):
        """Calculate the average 'height' of the stones of one colour.

        This is a silly example to demonstrate how to access the board position.

        """
        if len(params) != 1:
            raise pyego.PyGtpSyntaxError
        try:
            colour = {'b' : BLACK,
                      'w' : WHITE,
                      'black' : BLACK,
                      'white' : WHITE}[params[0].lower()]
        except KeyError:
            raise pyego.PyGtpSyntaxError

        board = self.stack.act_board()
        count = 0
        total_alt = 0
        for row in range (BOARD_SIZE):
            row_alt = min(row, BOARD_SIZE-row-1)
            for col in range(BOARD_SIZE):
                if board.point(Vertex(row, col)) == colour:
                    col_alt = min(col, BOARD_SIZE-col-1)
                    total_alt += min(row_alt, col_alt)
                    count += 1
        if count == 0:
            return "n/a"
        return "%d stones, average altitude %.2f" % (count, total_alt/count + 1)

def gtp_demo():
    stack_board = pyego.StackBoard()
    uct = pyego.Uct(stack_board)

    gtp = pyego.Gtp()

    sc = pyego.GtpStaticCommands()
    sc.add("protocol_version", "2")
    sc.add("name", "libEGO-pyego")
    gtp.register_engine (sc)

    gtp_board = pyego.GtpBoard(stack_board)
    gtp.register_engine(gtp_board)

    extras = DemoCommands(stack_board)
    gtp.register_engine(extras)

    gtp_genmove = pyego.GtpGenmove(stack_board, uct)
    gtp.register_engine(gtp_genmove)

    gtp.run_loop()

if __name__ == "__main__":
    gtp_demo()

