// A gtp_engine which calls back to Python to implement its commands.

// This is implemented directly with the Python C API, rather than Pyrex.

class gtp_py_t : public gtp_engine_t {

public:

    PyObject *pyengine;
    PyObject *gtp_error_exc;
    PyObject *gtp_syntax_error_exc;

    gtp_py_t(PyObject *pyengine_,
	     PyObject* gtp_error_exc_,
	     PyObject* gtp_syntax_error_exc_) {
	// No need to reference count this, as it is our 'parent'.
	pyengine = pyengine_;
	// The parent keeps references to these for us.
	gtp_error_exc = gtp_error_exc_;
	gtp_syntax_error_exc = gtp_syntax_error_exc_;
    }

    virtual vector <string> get_command_names() const {
	PyObject *cmds_seq = 0;
	PyObject *iterator = 0;
	PyObject *item = 0;
	char *cstr = 0;
	vector <string> commands;

	cmds_seq = PyObject_CallMethod(pyengine, "get_command_names", NULL);
	if (!cmds_seq) goto fail;

	iterator = PyObject_GetIter(cmds_seq);
	if (!iterator) goto fail;
	while ((item = PyIter_Next(iterator))) {
	    cstr = PyString_AsString(item);
	    if (!cstr) goto fail;
	    commands.push_back(cstr);
	    Py_DECREF(item);
	    item = 0;
	}
	if (PyErr_Occurred()) goto fail;
	Py_DECREF(iterator);
	Py_DECREF(cmds_seq);
	return commands;

    fail:
	PyErr_Print();
	cerr << "ignoring commands from Python GTP engine\n";
	Py_XDECREF(cmds_seq);
	Py_XDECREF(iterator);
	Py_XDECREF(item);
	return vector <string>();
    };

    virtual gtp_status_t exec_command(string command,
				      istream& params, ostream& response) {
	gtp_status_t gtp_status = gtp_success;
	PyObject *prms = 0;
	PyObject *prm = 0;
	PyObject *response_s = 0;
	char * cstr = 0;
	string s;

	prms = PyList_New(0);
	if (!prms) goto fail;
	while (true) {
	    if (!(params >> s)) break;
	    prm = PyString_FromString(s.c_str());
	    if (!prm) goto fail;
	    if (PyList_Append(prms, prm)) goto fail;
	    Py_DECREF(prm);
	    prm = 0;
	}
	response_s = PyObject_CallMethod(
	    pyengine, "exec_command", "sO", command.c_str(), prms, NULL);
	if (response_s) {
	    cstr = PyString_AsString(response_s);
	    if (!cstr) goto fail;
	    response << cstr;
	} else {
	    if (PyErr_ExceptionMatches(gtp_syntax_error_exc)) {
		PyErr_Clear();
		gtp_status = gtp_syntax_error;
	    } else if (PyErr_ExceptionMatches(gtp_error_exc)) {
		PyObject *ptype;
		PyObject *pvalue;
		PyObject *ptraceback;
		PyObject *value_str;
		PyErr_Fetch(&ptype, &pvalue, &ptraceback);
		PyErr_Clear();
		value_str = PyObject_Str(pvalue);
		if (value_str) {
		    cstr = PyString_AsString(value_str);
		}
		if (cstr && *cstr) {
		    response << "error from command handler: " << cstr;
		} else {
		    response << "error from command handler";
		}
		Py_XDECREF(value_str);
		Py_DECREF(ptype);
		Py_DECREF(pvalue);
		Py_DECREF(ptraceback);
		gtp_status = gtp_failure;
	    } else {
		goto fail;
	    }
	}

	Py_DECREF(prms);
	Py_XDECREF(response_s);
	return gtp_status;

    fail:
	PyErr_Print();
	response << "internal error from Python code";
	Py_XDECREF(prms);
	Py_XDECREF(prm);
	Py_XDECREF(response_s);
	return gtp_failure;
    }

};

